//lcd 0x27 - amp 0x4b - tone 0x44
#include <Wire.h>
#include <EEPROM.h>


//LED Outputs
#define LED1 2
#define LED2 3
#define LED3 4
#define LED4 5

//Button Inputs
#define BUTTON1 A0
#define BUTTON2 A1
#define BUTTON3 A2
#define BUTTON4 A3

//Control lines to NJM2744
#define CTRL1 7
#define CTRL2 6

//I2C address inputs
#define ADDR0 8
#define ADDR1 9

#define DEBOUNCE 50

#define I2CADDRESS 0x2C


byte INPUTMODE = 0; //0 = Rotary switch - 1 = Push Buttons
byte requestedInput = 0;
byte lastInput = 4; // default to invalid input to ensure updte on start
//byte buttonPressed = 0;
boolean ledsOn = true; // should the LED's be lit
boolean useRGBLED = false; // using single or RGB leds


void setup() {
  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  pinMode(LED3, OUTPUT);
  pinMode(LED4, OUTPUT);

  pinMode(CTRL1, OUTPUT);
  pinMode(CTRL2, OUTPUT);

  pinMode(BUTTON1, INPUT_PULLUP);
  pinMode(BUTTON2, INPUT_PULLUP);
  pinMode(BUTTON3, INPUT_PULLUP);
  pinMode(BUTTON4, INPUT_PULLUP);

  pinMode(ADDR0, INPUT_PULLUP);
  pinMode(ADDR1, INPUT_PULLUP);


  requestedInput = EEPROM.read(0);





  //scan buttons, if any are low then a rotary switch is connected and start in rotary mode, otherwise start in push button/I2C mode
  if (digitalRead(BUTTON1) == 0 || digitalRead(BUTTON2) == 0 || digitalRead(BUTTON3) == 0 || digitalRead(BUTTON4) == 0) {
    INPUTMODE = 0;
    digitalWrite(LED1, HIGH); // flash leds to show rotary mode selected
    delay(100);
    digitalWrite(LED1, LOW);
    digitalWrite(LED2, HIGH);
    delay(100);
    digitalWrite(LED2, LOW);
    digitalWrite(LED3, HIGH);
    delay(100);
    digitalWrite(LED3, LOW);
    digitalWrite(LED4, HIGH);
    delay(100);
    digitalWrite(LED4, LOW);
  }
  else {
    INPUTMODE = 1;

    inputSelect(requestedInput);

    byte readAddress = 0; // temp byte for read address offset - This reads the two bits and sets up an
    if (digitalRead(ADDR0) == LOW) bitSet(readAddress, 0); // offset to be added to the base i2c address
    if (digitalRead(ADDR1) == LOW) bitSet(readAddress, 1); // set in the #define above


    Wire.begin(I2CADDRESS + readAddress); // add offset and start the i2c service

    Wire.onRequest(requestEvent); // this sets the response when the host requests for the current input selected
    Wire.onReceive(receiveEvent);

    digitalWrite(LED4, HIGH); // flash leds to show i2c/push button mode
    delay(100);
    digitalWrite(LED4, LOW);
    digitalWrite(LED3, HIGH);
    delay(100);
    digitalWrite(LED3, LOW);
    digitalWrite(LED2, HIGH);
    delay(100);
    digitalWrite(LED2, LOW);
    digitalWrite(LED1, HIGH);
    delay(100);
    digitalWrite(LED1, LOW);
  }

  delay(100);


}

void loop() {

  /*
     The next set of if statments scan the button inputs. If one is found to be active (pulled low) then
     the input may need changing. The requestedInput stores which button is pressed.
     If in i2c mode then the system will pause until the button is released.
  */
  if (digitalRead(BUTTON1) == 0) {
    requestedInput = 0;
    while (digitalRead(BUTTON1) == 0 && INPUTMODE == 1) {}
    delay(DEBOUNCE);
  }
  else if (digitalRead(BUTTON2) == 0) {
    requestedInput = 1;
    while (digitalRead(BUTTON2) == 0 && INPUTMODE == 1) {}
    delay(DEBOUNCE);
  }
  else if (digitalRead(BUTTON3) == 0) {
    requestedInput = 2;
    while (digitalRead(BUTTON3) == 0 && INPUTMODE == 1) {}
    delay(DEBOUNCE);
  }
  else if (digitalRead(BUTTON4) == 0) {
    requestedInput = 3;
    while (digitalRead(BUTTON4) == 0 && INPUTMODE == 1) {}
    delay(DEBOUNCE);
  }



  /*
     If the system is set in button/i2c mode then this will check if the host has sent through a
     change request and switch to it

      D7-3 Not Used
      D2   0=Turn off LEDs | 1=Turn on LEDs
      D1-0 Input Selection
  */
  //  if (INPUTMODE == 1) {
  //    if (Wire.available() > 0) {                   // data waiting on i2c bus
  //      byte in = Wire.read();                      // read in the single byte
  //      Serial.println(in, DEC);
  //      if (bitRead(in, 2) == HIGH) ledsOn = true;  // if bit 2 is set then allow leds to illuminate
  //      else ledsOn = false;                        // otherwise turn leds off
  //      requestedInput = in & B00000011;            // strip out all but the last to bits to determin input requested
  //    }
  //  }



  /*
     When the requested input is different then change the NJM chip to the
     selected input and illuminate the correct LED.
  */
  if (requestedInput != lastInput) {
    if (requestedInput == 0) {
      inputSelect(0);
      digitalWrite(LED1, HIGH);
      digitalWrite(LED2, LOW);
      digitalWrite(LED3, LOW);
      digitalWrite(LED4, LOW);
    }
    else if (requestedInput == 1) {
      inputSelect(1);
      digitalWrite(LED1, LOW);
      digitalWrite(LED2, HIGH);
      digitalWrite(LED3, LOW);
      digitalWrite(LED4, LOW);
    }
    else if (requestedInput == 2) {
      inputSelect(2);
      digitalWrite(LED1, LOW);
      digitalWrite(LED2, LOW);
      digitalWrite(LED3, HIGH);
      digitalWrite(LED4, LOW);
    }
    else if (requestedInput == 3) {
      inputSelect(3);
      digitalWrite(LED1, LOW);
      digitalWrite(LED2, LOW);
      digitalWrite(LED3, LOW);
      digitalWrite(LED4, HIGH);
    }
    //    delay(DEBOUNCE);
    EEPROM.update(0, requestedInput);
  }

  lastInput = requestedInput;

}



/*
   The function will set the correct pins to select the correct input on the chip
*/
void inputSelect(byte i) {
  if (i == 0)      {
    digitalWrite(CTRL1, LOW);
    digitalWrite(CTRL2 , LOW);
  }
  else if (i == 1) {
    digitalWrite(CTRL1, HIGH);
    digitalWrite(CTRL2 , LOW);
  }
  else if (i == 2) {
    digitalWrite(CTRL1, LOW);
    digitalWrite(CTRL2 , HIGH);
  }
  else if (i == 3) {
    digitalWrite(CTRL1, HIGH);
    digitalWrite(CTRL2 , HIGH);
  }
}




/*
   Event handler for when the host requests which input is currently selected
*/
void requestEvent() {
  byte outByte = requestedInput;
  Wire.write(outByte);
}





void receiveEvent(int howMany) {
  if (INPUTMODE == 1) {
    if (Wire.available() > 0) {                   // data waiting on i2c bus
      byte in = Wire.read();                      // read in the single byte
      if (bitRead(in, 2) == HIGH) ledsOn = true;  // if bit 2 is set then allow leds to illuminate
      else ledsOn = false;                        // otherwise turn leds off
      requestedInput = in & B00000011;            // strip out all but the last to bits to determin input requested
    }
  }
}
